@@ -17,7 +17,7 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-from riak import RiakError 
+from riak import RiakError
 import base64
 import random
 import threading
@@ -76,6 +76,13 @@ class RiakTransport(object):
         """
         raise RiakError("not implemented")
 
+    def get_buckets(self) :
+        """
+        Serialize get buckets request and deserialize response
+        @return dict()
+        """
+        raise RiakError("not implemented")
+
     def get_bucket_props(self, bucket) :
         """
         Serialize get bucket property request and deserialize response
@@ -100,3 +107,4 @@ class RiakTransport(object):
 
     def set_client_id(self, client_id):
         raise RiakError("not implemented")
+
